import { RabbitMQModule } from '@golevelup/nestjs-rabbitmq';
import { Module } from '@nestjs/common';
import { ConfigModule, ConfigService } from '@nestjs/config';
import { CqrsModule } from '@nestjs/cqrs';
import { DatabaseModule } from '../database/database.module';
import { ConfigurationController } from './adapters/primaries/configuration.controller';
import { Messager } from './adapters/secondaries/messager';
import { ConfigurationRepository } from './adapters/secondaries/configuration.repository';
import { CreateConfigurationUseCase } from './domain/usecases/create-configuration.usecase';
import { DeleteConfigurationUseCase } from './domain/usecases/delete-configuration.usecase';
import { FindAllConfigurationsUseCase } from './domain/usecases/find-all-configurations.usecase';
import { FindConfigurationByUuidUseCase } from './domain/usecases/find-configuration-by-uuid.usecase';
import { PropagateConfigurationsUseCase } from './domain/usecases/propagate-configurations.usecase';
import { UpdateConfigurationUseCase } from './domain/usecases/update-configuration.usecase';
import { ConfigurationProfile } from './mappers/configuration.profile';

@Module({
  imports: [
    DatabaseModule,
    CqrsModule,
    RabbitMQModule.forRootAsync(RabbitMQModule, {
      imports: [ConfigModule],
      inject: [ConfigService],
      useFactory: async (configService: ConfigService) => ({
        exchanges: [
          {
            name: configService.get<string>('RMQ_EXCHANGE'),
            type: 'topic',
          },
        ],
        uri: configService.get<string>('RMQ_URI'),
        connectionInitOptions: { wait: false },
      }),
    }),
  ],
  exports: [],
  controllers: [ConfigurationController],
  providers: [
    ConfigurationProfile,
    ConfigurationRepository,
    Messager,
    FindAllConfigurationsUseCase,
    FindConfigurationByUuidUseCase,
    CreateConfigurationUseCase,
    UpdateConfigurationUseCase,
    DeleteConfigurationUseCase,
    PropagateConfigurationsUseCase,
  ],
})
export class ConfigurationModule {}
