import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { CommandHandler } from '@nestjs/cqrs';
import { ConfigurationMessagerPresenter } from '../../adapters/secondaries/configuration-messager.presenter';
import { Messager } from '../../adapters/secondaries/messager';
import { ConfigurationRepository } from '../../adapters/secondaries/configuration.repository';
import { CreateConfigurationCommand } from '../../commands/create-configuration.command';
import { CreateConfigurationRequest } from '../dtos/create-configuration.request';
import { Configuration } from '../entities/configuration';

@CommandHandler(CreateConfigurationCommand)
export class CreateConfigurationUseCase {
  constructor(
    private readonly repository: ConfigurationRepository,
    private readonly messager: Messager,
    @InjectMapper() private readonly mapper: Mapper,
  ) {}

  execute = async (
    command: CreateConfigurationCommand,
  ): Promise<Configuration> => {
    const entity = this.mapper.map(
      command.createConfigurationRequest,
      CreateConfigurationRequest,
      Configuration,
    );

    try {
      const configuration = await this.repository.create(entity);
      this.messager.publish(
        'configuration.create',
        JSON.stringify(
          this.mapper.map(
            configuration,
            Configuration,
            ConfigurationMessagerPresenter,
          ),
        ),
      );
      this.messager.publish(
        'logging.configuration.create.info',
        JSON.stringify(configuration),
      );
      return configuration;
    } catch (error) {
      let key = 'logging.configuration.create.crit';
      if (error.message.includes('Already exists')) {
        key = 'logging.configuration.create.warning';
      }
      this.messager.publish(
        key,
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  };
}
