import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { CommandHandler } from '@nestjs/cqrs';
import { ConfigurationMessagerPresenter } from '../../adapters/secondaries/configuration-messager.presenter';
import { Messager } from '../../adapters/secondaries/messager';
import { ConfigurationRepository } from '../../adapters/secondaries/configuration.repository';
import { DeleteConfigurationCommand } from '../../commands/delete-configuration.command';
import { Configuration } from '../entities/configuration';

@CommandHandler(DeleteConfigurationCommand)
export class DeleteConfigurationUseCase {
  constructor(
    private readonly repository: ConfigurationRepository,
    private readonly messager: Messager,
    @InjectMapper() private readonly mapper: Mapper,
  ) {}

  execute = async (
    command: DeleteConfigurationCommand,
  ): Promise<Configuration> => {
    try {
      const configuration = await this.repository.delete(command.uuid);
      this.messager.publish(
        'configuration.delete',
        JSON.stringify(
          this.mapper.map(
            configuration,
            Configuration,
            ConfigurationMessagerPresenter,
          ),
        ),
      );
      this.messager.publish(
        'logging.configuration.delete.info',
        JSON.stringify({ uuid: configuration.uuid }),
      );
      return configuration;
    } catch (error) {
      this.messager.publish(
        'logging.configuration.delete.crit',
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  };
}
