import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { QueryHandler } from '@nestjs/cqrs';
import { ConfigurationMessagerPresenter } from '../../adapters/secondaries/configuration-messager.presenter';
import { Messager } from '../../adapters/secondaries/messager';
import { ConfigurationRepository } from '../../adapters/secondaries/configuration.repository';
import { PropagateConfigurationsQuery } from '../../queries/propagate-configurations.query';
import { Configuration } from '../entities/configuration';

@QueryHandler(PropagateConfigurationsQuery)
export class PropagateConfigurationsUseCase {
  constructor(
    private readonly repository: ConfigurationRepository,
    private readonly messager: Messager,
    @InjectMapper() private readonly mapper: Mapper,
  ) {}

  execute = async (
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    propagateConfigurationsQuery: PropagateConfigurationsQuery,
  ) => {
    try {
      const configurations = await this.repository.findAll(1, 999999);
      this.messager.publish(
        'configuration.propagate',
        JSON.stringify(
          configurations.data.map((configuration) =>
            this.mapper.map(
              configuration,
              Configuration,
              ConfigurationMessagerPresenter,
            ),
          ),
        ),
      );
      this.messager.publish('logging.configuration.update.info', 'propagation');
    } catch (error) {
      this.messager.publish('logging.configuration.update.crit', 'propagation');
      throw error;
    }
  };
}
