import { classes } from '@automapper/classes';
import { AutomapperModule } from '@automapper/nestjs';
import { Test, TestingModule } from '@nestjs/testing';
import { ICollection } from 'src/modules/database/interfaces/collection.interface';
import { Messager } from '../../adapters/secondaries/messager';
import { ConfigurationRepository } from '../../adapters/secondaries/configuration.repository';
import { Domain } from '../../domain/dtos/domain.enum';
import { Configuration } from '../../domain/entities/configuration';
import { PropagateConfigurationsUseCase } from '../../domain/usecases/propagate-configurations.usecase';
import { ConfigurationProfile } from '../../mappers/configuration.profile';
import { PropagateConfigurationsQuery } from '../../queries/propagate-configurations.query';

const propagateConfigurationsQuery: PropagateConfigurationsQuery =
  new PropagateConfigurationsQuery();

const mockConfigurations: ICollection<Configuration> = {
  data: [
    {
      uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
      domain: Domain.USER,
      key: 'key1',
      value: 'value1',
    },
    {
      uuid: 'bb281075-1b98-4456-89d6-c643d3044a92',
      domain: Domain.USER,
      key: 'key2',
      value: 'value2',
    },
    {
      uuid: 'bb281075-1b98-4456-89d6-c643d3044a93',
      domain: Domain.USER,
      key: 'key3',
      value: 'value3',
    },
  ],
  total: 3,
};

const mockConfigurationRepository = {
  findAll: jest
    .fn()
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    .mockImplementationOnce((query?: PropagateConfigurationsQuery) => {
      return Promise.resolve(mockConfigurations);
    })
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    .mockImplementationOnce((query?: PropagateConfigurationsQuery) => {
      throw new Error();
    }),
};

const mockMessager = {
  publish: jest.fn().mockImplementation(),
};

describe('PropagateConfigurationsUseCase', () => {
  let propagateConfigurationsUseCase: PropagateConfigurationsUseCase;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [AutomapperModule.forRoot({ strategyInitializer: classes() })],
      providers: [
        {
          provide: ConfigurationRepository,
          useValue: mockConfigurationRepository,
        },
        {
          provide: Messager,
          useValue: mockMessager,
        },
        PropagateConfigurationsUseCase,
        ConfigurationProfile,
      ],
    }).compile();

    propagateConfigurationsUseCase = module.get<PropagateConfigurationsUseCase>(
      PropagateConfigurationsUseCase,
    );
  });

  it('should be defined', () => {
    expect(propagateConfigurationsUseCase).toBeDefined();
  });

  describe('execute', () => {
    it('should propagate configurations', async () => {
      jest.spyOn(mockMessager, 'publish');
      await propagateConfigurationsUseCase.execute(
        propagateConfigurationsQuery,
      );
      expect(mockMessager.publish).toHaveBeenCalledTimes(2);
    });
    it('should throw an error if repository call fails', async () => {
      await expect(
        propagateConfigurationsUseCase.execute(propagateConfigurationsQuery),
      ).rejects.toBeInstanceOf(Error);
    });
  });
});
